var path = require('path');

module.exports = function (grunt) {
    grunt.loadNpmTasks('grunt-contrib-jshint');
    grunt.loadNpmTasks('grunt-contrib-concat');
    grunt.loadNpmTasks('grunt-contrib-uglify');
    grunt.loadNpmTasks('grunt-contrib-connect');
    grunt.loadNpmTasks('grunt-contrib-qunit');
    grunt.loadNpmTasks('grunt-contrib-yuidoc');

    var srcFiles = [
        '<%= dirs.src %>/Intro.js',
        '<%= dirs.src %>/Link.js',
        '<%= dirs.src %>/Utils.js',
        '<%= dirs.src %>/Layers.js',
        '<%= dirs.src %>/Keyboard.js',
        '<%= dirs.src %>/Mouse.js',
        '<%= dirs.src %>/Camera.js',
        '<%= dirs.src %>/Network.js',
        '<%= dirs.src %>/SimpleGrid.js',
        '<%= dirs.src %>/Stats.js',
        '<%= dirs.src %>/Utils.js',
        '<%= dirs.src %>/MovieClipManager.js',
        '<%= dirs.src %>/loaders/JsonLoader.js',
        '<%= dirs.src %>/loaders/Loader.js',
        '<%= dirs.src %>/TiledMap.js',
        '<%= dirs.src %>/Outro.js'],
        vendorFiles = [
            '<%= dirs.vendor %>/stats.js',
            '<%= dirs.vendor %>/msgpack.js',
            '<%= dirs.vendor %>/es5-shim.min.js',
            '<%= dirs.vendor %>/es5-sham.min.js',
            '<%= dirs.vendor %>/pixi.dev.js'],
        banner = [
            '/**',
            ' * @license',
            ' * <%= pkg.longName %> - v<%= pkg.version %>',
            ' * Copyright (c) 2013, Vsevolod Strukchinsky',
            ' * <%= pkg.homepage %>',
            ' *',
            ' * Compiled: <%= grunt.template.today("yyyy-mm-dd") %>',
            ' *',
            ' * <%= pkg.longName %> is licensed under the <%= pkg.license %> License.',
            ' * <%= pkg.licenseUrl %>',
            ' */',
            ''].join('\n');

    //Project Configuration
    grunt.initConfig({
        pkg: grunt.file.readJSON('package.json'),
        dirs: {
            build: 'build',
            docs: 'docs',
            src: 'src/link',
            test: 'test',
            vendor: 'vendor'
        },
        files: {
            vendorBlob: '<%= dirs.vendor %>/**/*js',
            srcBlob: '<%= dirs.src %>/**/*.js',
            testBlob: '<%= dirs.test %>/unit/**/*.js',
            build: '<%= dirs.build %>/<%= pkg.name %>.js',
            buildMin: '<%= dirs.build %>/<%= pkg.name %>.min.js'
        },
        concat: {
            options: {
                banner: banner
            },
            dist: {
                src: vendorFiles.concat(srcFiles),
                dest: '<%= files.build %>'
            }
        },
        uglify: {
            options: {
                banner: banner
            },
            dist: {
                src: '<%= files.build %>',
                dest: '<%= files.buildMin %>'
            }
        },
        jshint: {
            beforeconcat: srcFiles.filter(function (e) {
                return e.indexOf('Intro.js') === -1 && e.indexOf('Link.js') === -1 && e.indexOf('Outro.js') === -1;
            }),
            test: ['<%= files.testBlob %>'],
            options: {
                /* Enforcement options */
                bitwise: false, //allow bitwise operators
                camelcase: false, //must use camelCase or UPPER_CASE
                curly: false, //one line conditionals w/o braces are allowed
                eqeqeq: true, //must use === if possible
                forin: false, //forin loops much check hasOwnProperty
                immed: true, //self-calling functions must be wrapped in parens
                latedef: true, //can't use a variable until it is defined
                newcap: true, //ctor names must be Captialized
                noarg: true, //arguments.caller/callee are deprecated, disallow
                noempty: true, //warn about empty blocks
                nonew: true, //no using `new Constructor();` without saving the value (no using only side-effects)
                plusplus: false, //you can use unary increment and decrement operators
                quotmark: true, //quotes must be consistent
                unused: true, //warn about declared but not used variables
                strict: false, //do not require functions to be able to run in strict-mode
                trailing: true, //help prevent weird whitespace errors in multi-line strings using \ 
                maxlen: 200, //no line should be longer than 120 characters

                /* Relaxing Options */
                boss: true, //do not warn about the use of assignments in cases where comparisons are expected

                /* Environments */
                browser: true, //this runs in a browser :)
                devel: false, //warn about using console.log and the like
                jquery: false, //no jquery used here
                node: false, //no node support...YET! :)
                worker: true, //web-workers are used

                /* Globals */
                undef: true,
                globals: {
                    /* For tests */
                    requirejs: false,
                    require: false,
                    define: false,
                    QUnit: false,
                    Q: false,
                    $: false,
                    LINK: false,

                    /* For the library */
                    PIXI: false
                }
            }
        },
        connect: {
            qunit: {
                options: {
                    port: grunt.option('port-test') || 9002,
                    base: './'
                }
            },
            test: {
                options: {
                    port: grunt.option('port-test') || 9002,
                    base: './',
                    keepalive: true
                }
            }
        },
        qunit: {
            all: {
                options: {
                    urls: ['http://localhost:' + (grunt.option('port-test') || 9002) + '/test/index.html']
                }
            }
        },
        yuidoc: {
            compile: {
                name: '<%= pkg.name %>',
                description: '<%= pkg.description %>',
                version: '<%= pkg.version %>',
                url: '<%= pkg.homepage %>',
                options: {
                    paths: '<%= dirs.src %>',
                    outdir: '<%= dirs.docs %>'
                }
            }
        }
    });

    //Load tasks
    grunt.registerTask('default', ['build', 'test']);
    grunt.registerTask('build', ['jshint', 'concat', 'uglify']);
    grunt.registerTask('test', ['connect:qunit', 'qunit']);
    grunt.registerTask('docs', ['yuidoc']);
};