/**
 * @author Vsevolod Strukchinsky @floatdrop
 */

/**
Mouse class handles clicks.

@example
	var mouse = new LINK.Mouse();
	mouse.on.click(shootCallback);
	mouse.on.move(lookCallback);

@class Mouse
@constructor
**/
LINK.Mouse = function (objectToListen) {

	var self = this;

	objectToListen = objectToListen || document;

	this.offset = LINK.Utils.getOffset(objectToListen);

	/**
	 * The current position of the mouse
	 *
	 * @property position
	 * @type Point
	 * @readOnly
	 */
	this.position = new PIXI.Point(0, 0);
	objectToListen.addEventListener('mousemove', this.onMouseMove.bind(this), false);

	this.callbacks = {};

	this.on = {};

	Object.keys(LINK.Mouse.prototype.Events).forEach(function (eventName) {
		self.on[eventName] = function (callback) {
			self.callbacks[self.Events[eventName]] = callback;
		};

		var event = LINK.Mouse.prototype.Events[eventName];
		if (event === 'mousemove' || event === 'mousewheel') return;
		objectToListen.addEventListener(event, self.onMouse.bind(self), false);

	});

};

LINK.Mouse.constructor = LINK.Mouse;

LINK.Mouse.prototype.onMouse = function (e) {
	this.updateCoords(e);
	if (e.type && this.callbacks[e.type]) {
		this.callbacks[e.type](e);
	}
	return true;
};

LINK.Mouse.prototype.onMouseMove = function (e) {
	this.updateCoords(e);
	if (this.callbacks.mousemove) {
		this.callbacks.mousemove(e);
	}
	return true;
};

LINK.Mouse.prototype.updateCoords = function (e) {
	this.position.x = e.pageX - this.offset.left;
	this.position.y = e.pageY - this.offset.top;
};

LINK.Mouse.prototype.Events = {
	wheel: 'mousewheel',
	move: 'mousemove',
	down: 'mousedown',
	up: 'mouseup',
	click: 'click',
	dbclick: 'dblclick',
	rclick: 'contextmenu',
	contextmenu: 'contextmenu'
};