/**
 * @author Vsevolod Strukchinsky @floatdrop
 */


/**
 *
 * @class SimpleGrid provides easy object storing in different DisplayObjectContainers
 * grouped by theirs position. It should simplify partial rendering and collisions.
 * @extends DisplayObjectContainer
 * @constructor
 */
LINK.SimpleGrid = function (widthPower, heightPower) {
	LINK.Layers.call(this);

	this._defaultVisibility = false;

	this._width = widthPower || 10;
	this._height = heightPower || 10;

	this.visibleCells = {};
};

LINK.SimpleGrid.constructor = LINK.SimpleGrid;
LINK.SimpleGrid.prototype = Object.create(LINK.Layers.prototype);


LINK.SimpleGrid.prototype.getCellСoordinates = function (displayObject) {
	return {
		x: displayObject.position.x >> this._width,
		y: displayObject.position.y >> this._height
	};
};

LINK.SimpleGrid.prototype.getCellName = function (displayObject) {
	var coords = this.getCellСoordinates(displayObject);
	return coords.x + "_" + coords.y;
};

/**
 * Add child to one of the grid cells
 * @method addChild
 * @param  displayObject {DisplayObject}
 * @return DisplayObject
 */
LINK.SimpleGrid.prototype.addChild = function (displayObject) {
	// Wrap position property of displayObject
	// Rewrap PIXI.Point object

	this.getLayer(this.getCellName(displayObject)).addChild(displayObject);
};

LINK.SimpleGrid.prototype.getSurroundCells = function (displayObject, bounds) {
	bounds = bounds || new PIXI.Rectangle(0, 0, 800, 600);
	var coords = this.getCellСoordinates(displayObject);
	var result = [];

	for (var x = coords.x - (bounds.width >> (this._width + 1)); x < coords.x + ((bounds.width * 2) >> (this._width + 1)); x++) {
		for (var y = coords.y - (bounds.height >> (this._height + 1)); y < coords.y + ((bounds.height * 2) >> (this._height + 1)); y++) {
			result.push(this[x + "_" + y]);
		}
	}

	return result;
};

LINK.SimpleGrid.prototype.setVisibleAround = function (displayObject, bounds, visibility) {
	var self = this;
	visibility = visibility || true;
	var surroundCells = this.getSurroundCells(displayObject, bounds);
	surroundCells.forEach(function (e, i) {
		if (!e) return;
		if (visibility) {
			self.visibleCells[i] = e;
		} else {
			self.visibleCells[i] = undefined;
		}
		e.visible = visibility;
	});
};