/**
 * @author Vsevolod Strukchinsky @floatdrop
 */

/**
 * @class JsonLoader
 * @extends EventTarget
 * @constructor
 * @param {String} url the url of the sprite sheet JSON file
 * @param {Boolean} crossorigin
 */

var AjaxRequest = function () {
	var activexmodes = ["Msxml2.XMLHTTP", "Microsoft.XMLHTTP"]; //activeX versions to check for in IE

	if (window.ActiveXObject) { //Test for support for ActiveXObject in IE first (as XMLHttpRequest in IE7 is broken)
		for (var i = 0; i < activexmodes.length; i++) {
			try {
				return new window.ActiveXObject(activexmodes[i]);
			} catch (e) {
				//suppress error
			}
		}
	} else if (window.XMLHttpRequest) // if Mozilla, Safari etc
	{
		return new window.XMLHttpRequest();
	} else {
		return false;
	}
};

LINK.JsonLoader = function (url, crossorigin) {
	PIXI.EventTarget.call(this);
	this.url = url;
	this.baseUrl = url.replace(/[^\/]*$/, "");
	this.json = null;
	this.crossorigin = crossorigin;
};

// constructor
LINK.JsonLoader.constructor = LINK.JsonLoader;

/**
 * This will begin loading the JSON file
 */
LINK.JsonLoader.prototype.load = function () {
	this.ajaxRequest = new AjaxRequest();
	var scope = this;
	this.ajaxRequest.onreadystatechange = function () {
		scope.onJSONLoaded();
	};

	this.ajaxRequest.open("GET", this.url, true);
	if (this.ajaxRequest.overrideMimeType) this.ajaxRequest.overrideMimeType("application/json");
	this.ajaxRequest.send(null);
};

/**
 * Invoke when JSON file is loaded
 * @private
 */
LINK.JsonLoader.prototype.onJSONLoaded = function () {
	if (this.ajaxRequest.readyState === 4) {
		if (this.ajaxRequest.status === 200 || window.location.href.indexOf("http") === -1) {
			this.json = JSON.parse(this.ajaxRequest.responseText);
			LINK.Cache[this.url] = this.json;
			this.onLoaded();
		}
	}
};
/**
 * Invoke when all files are loaded (json and texture)
 * @private
 */
LINK.JsonLoader.prototype.onLoaded = function () {
	this.dispatchEvent({
		type: "loaded",
		content: this
	});
};